@@ -33,7 +33,7 @@ from jupyter_core.paths import jupyter_data_dir, jupyter_runtime_dir
 
 def write_connection_file(fname=None, shell_port=0, iopub_port=0, stdin_port=0, hb_port=0,
                          control_port=0, ip='', key=b'', transport='tcp',
-                         signature_scheme='hmac-sha256',
+                         signature_scheme='hmac-sha256', kernel_name=''
                          ):
     """Generates a JSON config file, including the selection of random ports.
 
@@ -72,6 +72,8 @@ def write_connection_file(fname=None, shell_port=0, iopub_port=0, stdin_port=0,
         Currently, 'hmac' is the only supported digest scheme,
         and 'sha256' is the default hash function.
 
+    kernel_name : str, optional
+        The name of the kernel currently connected to.
     """
     if not ip:
         ip = localhost()
@@ -93,7 +95,7 @@ def write_connection_file(fname=None, shell_port=0, iopub_port=0, stdin_port=0,
             sock = socket.socket()
             # struct.pack('ii', (0,0)) is 8 null bytes
             sock.setsockopt(socket.SOL_SOCKET, socket.SO_LINGER, b'\0' * 8)
-            sock.bind(('', 0))
+            sock.bind((ip, 0))
             ports.append(sock)
         for i, sock in enumerate(ports):
             port = sock.getsockname()[1]
@@ -127,6 +129,7 @@ def write_connection_file(fname=None, shell_port=0, iopub_port=0, stdin_port=0,
     cfg['key'] = bytes_to_str(key)
     cfg['transport'] = transport
     cfg['signature_scheme'] = signature_scheme
+    cfg['kernel_name'] = kernel_name
 
     with open(fname, 'w') as f:
         f.write(json.dumps(cfg, indent=2))
@@ -320,9 +323,22 @@ class ConnectionFileMixin(LoggingConfigurable):
     # Connection and ipc file management
     #--------------------------------------------------------------------------
 
-    def get_connection_info(self):
-        """return the connection info as a dict"""
-        return dict(
+    def get_connection_info(self, session=False):
+        """Return the connection info as a dict
+
+        Parameters
+        ----------
+        session : bool [default: False]
+            If True, return our session object will be included in the connection info.
+            If False (default), the configuration parameters of our session object will be included,
+            rather than the session object itself.
+
+        Returns
+        -------
+        connect_info : dict
+            dictionary of connection information.
+        """
+        info = dict(
             transport=self.transport,
             ip=self.ip,
             shell_port=self.shell_port,
@@ -330,9 +346,17 @@ class ConnectionFileMixin(LoggingConfigurable):
             stdin_port=self.stdin_port,
             hb_port=self.hb_port,
             control_port=self.control_port,
-            signature_scheme=self.session.signature_scheme,
-            key=self.session.key,
         )
+        if session:
+            # add session
+            info['session'] = self.session
+        else:
+            # add session info
+            info.update(dict(
+                signature_scheme=self.session.signature_scheme,
+                key=self.session.key,
+            ))
+        return info
 
     # factory for blocking clients
     blocking_class = Type(klass=object, default_value='jupyter_client.BlockingKernelClient')
@@ -379,6 +403,7 @@ class ConnectionFileMixin(LoggingConfigurable):
             shell_port=self.shell_port, hb_port=self.hb_port,
             control_port=self.control_port,
             signature_scheme=self.session.signature_scheme,
+            kernel_name=self.kernel_name
         )
         # write_connection_file also sets default ports:
         for name in port_names:
