@@ -12,6 +12,7 @@ try:
     from queue import Empty
 except ImportError:
     from Queue import Empty
+import re
 
 from qtconsole import qt
 from qtconsole.qt import QtCore, QtGui
@@ -25,6 +26,33 @@ from .call_tip_widget import CallTipWidget
 from .history_console_widget import HistoryConsoleWidget
 from .pygments_highlighter import PygmentsHighlighter
 
+_classic_prompt_re = re.compile(r'^([ \t]*>>> |^[ \t]*\.\.\. )')
+
+def transform_classic_prompt(line):
+    """Handle inputs that start with '>>> ' syntax."""
+
+    if not line or line.isspace():
+        return line
+    m = _classic_prompt_re.match(line)
+    if m:
+        return line[len(m.group(0)):]
+    else:
+        return line
+
+
+_ipy_prompt_re = re.compile(r'^([ \t]*In \[\d+\]: |^[ \t]*\ \ \ \.\.\.+: )')
+
+def transform_ipy_prompt(line):
+    """Handle inputs that start classic IPython prompt syntax."""
+
+    if not line or line.isspace():
+        return line
+    m = _ipy_prompt_re.match(line)
+    if m:
+        return line[len(m.group(0)):]
+    else:
+        return line
+
 
 class FrontendHighlighter(PygmentsHighlighter):
     """ A PygmentsHighlighter that understands and ignores prompts.
@@ -213,8 +241,13 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         elif self._control.hasFocus():
             text = self._control.textCursor().selection().toPlainText()
             if text:
+                # Remove prompts.
+                lines = text.splitlines()
+                lines = map(transform_classic_prompt, lines)
+                lines = map(transform_ipy_prompt, lines)
+                text = '\n'.join(lines)
                 was_newline = text[-1] == '\n'
-                if not was_newline: # user doesn't need newline
+                if was_newline:  # user doesn't need newline
                     text = text[:-1]
                 QtGui.QApplication.clipboard().setText(text)
         else:
@@ -433,6 +466,7 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         # unset reading flag, because if execute finished, raw_input can't
         # still be pending.
         self._reading = False
+        # Note:  If info is NoneType, this is ignored
         if info and info.kind == 'user' and not self._hidden:
             # Make sure that all output from the SUB channel has been processed
             # before writing a new prompt.
@@ -459,8 +493,8 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         elif info and info.kind == 'silent_exec_callback' and not self._hidden:
             self._handle_exec_callback(msg)
             self._request_info['execute'].pop(msg_id)
-        else:
-            super(FrontendWidget, self)._handle_execute_reply(msg)
+        elif info and not self._hidden:
+            raise RuntimeError("Unknown handler for %s" % info.kind)
 
     def _handle_input_request(self, msg):
         """ Handle requests for raw_input.
@@ -536,7 +570,7 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
     def _handle_shutdown_reply(self, msg):
         """ Handle shutdown signal, only if from other console.
         """
-        self.log.info("shutdown: %s", msg.get('content', ''))
+        self.log.debug("shutdown: %s", msg.get('content', ''))
         restart = msg.get('content', {}).get('restart', False)
         if not self._hidden and not self.from_here(msg):
             # got shutdown reply, request came from session other than ours
